"use strict";
const tslib_1 = require("tslib");
const Path = tslib_1.__importStar(require("path"));
const fs = tslib_1.__importStar(require("fs"));
const lodash_1 = tslib_1.__importDefault(require("lodash"));
const assert_1 = tslib_1.__importDefault(require("assert"));
const default_compose_opts_1 = tslib_1.__importDefault(require("./default-compose-opts"));
const provider_types_1 = tslib_1.__importDefault(require("./provider-types"));
const process_config_1 = tslib_1.__importDefault(require("./process-config"));
const util_1 = tslib_1.__importDefault(require("./util"));
/* eslint-disable max-statements */
function composeConfigDir(dir, data, options) {
    const exts = options.extSearch;
    const handlers = options.extHandlers;
    dir = Path.resolve(dir);
    const load = (key, provider) => {
        const found = exts
            .map((ext) => {
            const fullF = Path.join(dir, `${provider.name}.${ext}`);
            if (fs.existsSync(fullF)) {
                assert_1.default(handlers[ext], `Config handler for extension ${ext} missing`);
                util_1.default.uMerge(data, handlers[ext](fullF));
                return ext;
            }
            return undefined;
        })
            .filter((x) => x);
        if (lodash_1.default.isEmpty(found)) {
            const msg = `Config provider ${key}: no file ${provider.name} of ` +
                `extensions ${exts} found in ${dir}`;
            if (provider.type === provider_types_1.default.required) {
                if (options.failMissing !== false) {
                    throw new Error(msg);
                }
            }
            else if (provider.type === provider_types_1.default.warn) {
                if (options.warnMissing !== false) {
                    console.error("WARNING:", msg); // eslint-disable-line
                }
            }
        }
    };
    const filterOff = (filter) => {
        // if it's an array any element of the array is falsy => filtered off
        // otherwise if itself is falsy => filtered off
        return lodash_1.default.isArray(filter) ? lodash_1.default.find(filter, (x) => !x) !== undefined : !filter;
    };
    const isEnable = (p) => {
        const x = p.filter;
        assert_1.default(p.type, "config provider type must be specified");
        return (p.type !== provider_types_1.default.disabled &&
            (p.name || p.handler) &&
            (x === undefined ? true : !filterOff(x)));
    };
    const num = (x) => {
        return lodash_1.default.isString(x) ? parseInt(x, 10) : x;
    };
    const checkNaN = (x) => {
        return isNaN(x) ? -1 : x; // eslint-disable-line
    };
    const order = (p) => checkNaN(num(p.order));
    const providers = options.providers;
    const getList = () => options.providerList || Object.keys(providers);
    const list = getList().filter((k) => isEnable(providers[k]));
    assert_1.default(list.length > 0, "config providers empty");
    list
        .sort((a, b) => order(providers[a]) - order(providers[b]))
        .forEach((k) => {
        const p = providers[k];
        if (p.handler) {
            if (options.verbose) {
                console.log("Confippet.compose: calling config provider:", k, JSON.stringify(p)); // eslint-disable-line
            }
            util_1.default.uMerge(data, p.handler());
        }
        else {
            if (options.verbose) {
                console.log("Confippet.compose: loading config provider:", k, JSON.stringify(p)); // eslint-disable-line
            }
            load(k, p);
        }
    });
    return data;
}
function composeConfig(options) {
    options = options || {};
    if (options.useDefaults !== false) {
        options = util_1.default.merge(default_compose_opts_1.default(), options);
    }
    process_config_1.default(options, options);
    const dirs = options.dirs || [options.dir];
    const data = {};
    dirs.forEach((dir) => {
        composeConfigDir(dir, data, options);
    });
    return data;
}
module.exports = composeConfig;
//# sourceMappingURL=compose-config.js.map