"use strict";
const tslib_1 = require("tslib");
const provider_types_1 = tslib_1.__importDefault(require("./provider-types"));
const util_1 = tslib_1.__importDefault(require("./util"));
const envOrder = 1000;
const confippetEnvOrder = 2000;
function defaultOpts() {
    return {
        dir: "config",
        extSearch: ["json", "yaml", "js", "ts"],
        extHandlers: require("./ext-handlers"),
        failMissing: true,
        warnMissing: true,
        verbose: false,
        providerList: undefined,
        //
        // {{deployment}} - resolves to "development" by default
        // {{instance}} - resolves to "" by default
        // {{hostname}} - resolves to "" by default
        // {{fullHostname}} - resolves to "" by default
        //
        // types: required, optional, warn, disabled
        //
        providers: {
            default: {
                name: "default",
                filter: ["{{defaultFilter}}"],
                type: "{{defaultType}}",
                order: 100
            },
            defaultInstance: {
                name: "default-{{instance}}",
                filter: ["{{defaultFilter}}", "{{instance}}"],
                type: provider_types_1.default.warn,
                order: 120
            },
            deployment: {
                name: "{{deployment}}",
                filter: ["{{defaultFilter}}"],
                type: "{{defaultType}}",
                order: 140
            },
            deploymentInstance: {
                name: "{{deployment}}-{{instance}}",
                filter: ["{{defaultFilter}}", "{{instance}}"],
                type: provider_types_1.default.warn,
                order: 160
            },
            hostname: {
                name: "{{hostname}}",
                filter: ["{{defaultFilter}}"],
                type: provider_types_1.default.optional,
                order: 180
            },
            hostnameInstance: {
                name: "{{hostname}}-{{instance}}",
                filter: ["{{defaultFilter}}", "{{hostname}}", "{{instance}}"],
                type: provider_types_1.default.optional,
                order: 200
            },
            hostnameDeployment: {
                name: "{{hostname}}-{{deployment}}",
                filter: ["{{defaultFilter}}", "{{hostname}}"],
                type: provider_types_1.default.optional,
                order: 220
            },
            hostnameDeploymentInstance: {
                name: "{{hostname}}-{{deployment}}-{{instance}}",
                filter: ["{{defaultFilter}}", "{{hostname}}", "{{instance}}"],
                type: provider_types_1.default.optional,
                order: 240
            },
            fullHostname: {
                name: "{{fullHostname}}",
                filter: ["{{defaultFilter}}", "{{fullHostname}}"],
                type: provider_types_1.default.optional,
                order: 260
            },
            fullHostnameInstance: {
                name: "{{fullHostname}}-{{instance}}",
                filter: ["{{defaultFilter}}", "{{fullHostname}}", "{{instance}}"],
                type: provider_types_1.default.warn,
                order: 280
            },
            fullHostnameDeployment: {
                name: "{{fullHostname}}-{{deployment}}",
                filter: ["{{defaultFilter}}", "{{fullHostname}}"],
                type: provider_types_1.default.optional,
                order: 300
            },
            fullHostnameDeploymentInstance: {
                name: "{{fullHostname}}-{{deployment}}-{{instance}}",
                filter: ["{{defaultFilter}}", "{{fullHostname}}", "{{instance}}"],
                type: provider_types_1.default.warn,
                order: 320
            },
            local: {
                name: "local",
                filter: ["{{defaultFilter}}"],
                type: provider_types_1.default.optional,
                order: 340
            },
            localInstance: {
                name: "local-{{instance}}",
                filter: ["{{defaultFilter}}", "{{instance}}"],
                type: provider_types_1.default.warn,
                order: 360
            },
            localDeployment: {
                name: "local-{{deployment}}",
                filter: ["{{defaultFilter}}"],
                type: provider_types_1.default.optional,
                order: 380
            },
            localDeploymentInstance: {
                name: "local-{{deployment}}-{{instance}}",
                filter: ["{{defaultFilter}}", "{{instance}}"],
                type: provider_types_1.default.warn,
                order: 400
            },
            env: {
                filter: ["{{defaultFilter}}"],
                type: provider_types_1.default.disabled,
                handler: function () {
                    const env = {};
                    Object.keys(process.env).forEach((k) => {
                        env[k] = process.env[k];
                    });
                    return { env };
                },
                order: envOrder
            },
            confippetEnv: {
                filter: ["{{defaultFilter}}"],
                type: provider_types_1.default.required,
                handler: function () {
                    const data = {};
                    Object.keys(process.env).forEach((k) => {
                        if (k === "NODE_CONFIG" || k.startsWith("CONFIPPET")) {
                            util_1.default.uMerge(data, JSON.parse(process.env[k]));
                        }
                    });
                    return data;
                },
                order: confippetEnvOrder
            }
        },
        context: {
            deployment: "development",
            instance: "",
            hostname: "",
            fullHostname: "",
            defaultType: provider_types_1.default.required,
            defaultFilter: "enabled" // NOTE: set this to "" or undefined to disable all default providers
        }
    };
}
module.exports = defaultOpts;
//# sourceMappingURL=default-compose-opts.js.map