"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadConfig = void 0;
const tslib_1 = require("tslib");
const store_1 = tslib_1.__importDefault(require("./store"));
const Path = tslib_1.__importStar(require("path"));
const preset_config_1 = tslib_1.__importDefault(require("./preset-config"));
const configs = {};
const CACHED = Symbol("confippet loadConfig cached config");
/**
 * Load config files from a directory.
 *
 * - the directory can contain multiple files, each has a partial config.  this function
 *   will load the files according to various environment settings and compose them
 *   into a single one.
 *
 * - Each config file in the directory can be JSON, JS, Yaml, or TS.  For JS or TS, if it's
 *   ES6 Module and has `default` export, then it's used as config partial, else the whole
 *   module will be used.
 *
 * - The result is cached using the full directory path
 *
 * @param options LoadOptions for loading config files
 * @param defaults default config values
 * @param refresh refresh the cached copy - **NOTE** this will clear any values user set in the config
 *
 * @returns returns config
 */
const loadConfig = (options, defaults, refresh) => {
    const cache = options.cache !== false;
    const cacheKey = Path.resolve(options.dir);
    const config = (cache && configs[cacheKey]) || store_1.default();
    if (!cache || !config[CACHED] || refresh) {
        config._$.reset();
        config._$.defaults(defaults);
        preset_config_1.default.load(config, { ...options });
        if (cache) {
            config[CACHED] = true;
            configs[cacheKey] = config;
        }
    }
    return config;
};
exports.loadConfig = loadConfig;
//# sourceMappingURL=load-config.js.map